import 'dart:convert';

import 'usermanagement/identity.dart';
import 'usermanagement/conflict_user.dart';
import 'usermanagement/user_update.dart';
import 'src/private_user.dart';
import 'src/native_bridge.dart';

class CurrentUser extends PrivateUser {
  CurrentUser.fromJson(Map<String, dynamic> json): super.fromJson(json);

  ///
  /// Requests a bulk change of properties for the current user.
  ///
  /// [userUpdate] New user details.
  ///
  Future updateDetails(UserUpdate update) {
    return NativeBridge.async(
        "CurrentUser.updateDetails", jsonEncode(update.toJson()));
  }

  ///
  /// Adds Identity for the specified provider.
  ///
  /// [identity] Identity to be added.
  /// [success]  A callback to indicate if this operation was successful.
  /// [conflict] Called if there was a conflict.
  /// [failure]  Called if operation failed.
  ///
  void addIdentity(Identity identity, Function success,
      Function(ConflictUser) onConflict, Function onError) {
    NativeBridge.async("CurrentUser.addIdentity", jsonEncode(identity.toJSON()))
        .then((value) => {
          if (value == null || jsonDecode(value)['result'] == '') {
            success()
          } else {
            onConflict(ConflictUser.fromJson(jsonDecode(value)))
          }})
        .catchError(onError);
  }

  ///
  /// Removes Identity for the specified provider.
  ///
  /// [providerId] The provider connected to an auth identity on the current user to remove.
  /// Valid providerIds are found in [IdentityProviderIds] in addition to custom providers.
  /// [callback]   A callback to indicate if this operation was successful.
  /// [failure]    Failure callback. Called in case of error.
  ///
  Future removeIdentity(String provider) {
    return NativeBridge.async("CurrentUser.removeIdentity", provider);
  }

  @override
  String toString() {
    return 'CurrentUser{${super.toString()}';
  }
}
